@@ -141,6 +141,19 @@ function destr(obj) {
     }
 }
 
+function destrDialog(dlg) {
+    if (RSettings.getQtVersion() >= 0x060000) {
+        // delete later in Qt 6
+        // immediate delete might cause unpredictable crashes in event processing:
+        dlg.deleteLater();
+    }
+    else {
+        if (isFunction(dlg.destroy)) {
+            dlg.destroy();
+        }
+    }
+}
+
 /**
  * Get pointer from shared pointer object using data() (Qt5) or implicitly (Qt6).
  */
@@ -161,7 +174,7 @@ function getPtr(p) {
 /**
  * Checks if the given object is undefined or null.
  *
- * \return true if the given object is defined and not null.
+ * \return false if the given object is defined and not null.
  * This method also supports shared pointers, i.e. obj.isNull()==true.
  */
 function isNull(obj) {
@@ -277,6 +290,15 @@ function isQWidget(obj) {
     return (isQObject(obj) && typeof(obj.show)==="function");
 }
 
+/**
+ * Checks if the given object is a valid QAction.
+ *
+ * \return true if the given object is a QAction.
+ */
+function isQAction(obj) {
+    return (isQObject(obj) && typeof(obj.actionGroup)==="function");
+}
+
 /**
  * \return true if the given object is a separator (a frame of type vertical line).
  */
@@ -681,6 +703,18 @@ function isPolylineEntity(obj) {
     }
 }
 
+/**
+ * \return true if the given object is a wipeout entity (RWipeoutEntity).
+ */
+function isWipeoutEntity(obj) {
+    if (RSettings.getQtVersion() < 0x060000) {
+        return isOfType(obj, RWipeoutEntity) || isOfType(obj, RWipeoutEntityPointer);
+    }
+    else {
+        return isOfType(obj, RWipeoutEntity);
+    }
+}
+
 /**
  * \return true if the given object is a geometrically closed polyline entity.
  */
@@ -1226,6 +1260,10 @@ function entityTypeToClass(type) {
         return "RTextEntity";
     case RS.EntityViewport:
         return "RViewportEntity";
+    case RS.EntityWipeout:
+        return "RWipeoutEntity";
+    case RS.EntityXRef:
+        return "RXRefEntity";
     default:
         return undefined;
     }
@@ -1286,9 +1324,9 @@ function entityTypeToString(type, plural) {
     case RS.EntityDimAngular:
         return plural ? qsTr("Angular Dimensions") : qsTr("Angular Dimension");
     case RS.EntityDimAngular2L:
-        return plural ? qsTr("Angular Dimensions (2 Line)") : qsTr("Angular Dimension (2 Line)");
+        return plural ? qsTr("Angular Dimensions (2 Lines)") : qsTr("Angular Dimension (2 Lines)");
     case RS.EntityDimAngular3P:
-        return plural ? qsTr("Angular Dimensions (3 Point)") : qsTr("Angular Dimension (3 Point)");
+        return plural ? qsTr("Angular Dimensions (3 Points)") : qsTr("Angular Dimension (3 Points)");
     case RS.EntityDimArcLength:
         return plural ? qsTr("Arc Dimension") : qsTr("Arc Dimension");
     case RS.EntityDimDiametric:
@@ -1333,6 +1371,10 @@ function entityTypeToString(type, plural) {
         return plural ? qsTr("Texts") : qsTr("Text");
     case RS.EntityViewport:
         return plural ? qsTr("Viewports") : qsTr("Viewport");
+    case RS.EntityWipeout:
+        return plural ? qsTr("Wipeouts") : qsTr("Wipeout");
+    case RS.EntityXRef:
+        return plural ? qsTr("External References") : qsTr("External Reference");
     case RS.EntityUnknown:
     default:
         return plural ? qsTr("Unknown Entities") : qsTr("Unknown Entity");
@@ -1430,6 +1472,10 @@ function getEntityType(typeName) {
         return RS.EntityText;
     case "Viewport":
         return RS.EntityViewport;
+    case "Wipeout":
+        return RS.EntityWipeout;
+    case "External Reference":
+        return RS.EntityXRef;
     default:
         return RS.ObjectUnknown;
     }
@@ -1491,7 +1537,19 @@ function objectFromPath(path) {
  */
 function getWidgetPath(widget) {
     var pw = widget.parentWidget();
-    var str = widget.objectName !== "" ? widget.objectName : widget.toString();
+    var str = "";
+    if (RSettings.getQtVersion()>=0x060000) {
+        if (widget.objectName !== "") {
+            str = widget.objectName;
+        }
+        else {
+            return getWidgetPath(pw);
+        }
+    }
+    else {
+        str = widget.objectName !== "" ? widget.objectName : widget.toString();
+    }
+
     if (isNull(pw)) {
         return str;
     }
@@ -1875,6 +1933,12 @@ Array.prototype.remove = function(val, compareFunction) {
     }
 };
 
+Array.prototype.removeAt = function(index) {
+    var part1 = this.slice(0, index);
+    var part2 = this.slice(index + 1);
+    return (part1.concat(part2));
+};
+
 /**
  * Removes all matches of the given value from the array.
  * \param The value(s) to remove.
@@ -1933,7 +1997,7 @@ Array.prototype.clone = function() {
  */
 Array.prototype.sortNumerical = function() {
     Array.prototype.sort.call(this, function(a,b) { return a - b });
-}
+};
 
 Array.alphaNumericalSorter = function(a, b) {
     return RS.compareAlphanumerical(a, b);
@@ -1975,6 +2039,32 @@ Array.prototype.filter = function(func, thisArg) {
     return res;
 };
 
+Array.prototype.flat = function(depth) {
+    // If no depth is specified, default to 1
+    if (depth === undefined) {
+        depth = 1;
+    }
+
+    // Recursively reduce sub-arrays to the specified depth
+    var flatten = function (arr, depth) {
+
+        // If depth is 0, return the array as-is
+        if (depth < 1) {
+            return arr.slice();
+        }
+
+        // Otherwise, concatenate into the parent array
+        return arr.reduce(function (acc, val) {
+            return acc.concat(Array.isArray(val) ? flatten(val, depth - 1) : val);
+        }, []);
+
+    };
+
+    return flatten(this, depth);
+
+};
+
+
 /**
  * Sorts this array with string items (case insensitive)
  */
@@ -2472,6 +2562,10 @@ function stringToDirectDistanceEntry(relativeZero, cursorPosition, str) {
  * Creates and returns a new entity based on the given shape.
  */
 function shapeToEntity(document, shape) {
+    if (isNull(shape)) {
+        return undefined;
+    }
+
     var s = shape;
     if (isFunction(s.data)) {
         s = s.data();
@@ -2892,42 +2986,6 @@ function addActionsToWidgets() {
             }
             var w = appWin.findChild(wn);
             if (!isNull(w)) {
-
-                // workaround for QTBUG-38256 (action not triggered for letter based shortcuts in SUB menus):
-                // workaround for QTBUG-57990 (action not triggered for letter based shortcuts in ALL menus):
-                /*
-                if ((RSettings.getQtVersion()<0x050500 && RSettings.getQtVersion()>=0x050000) ||
-                    (RSettings.getQtVersion()===0x050800 && RS.getSystemId()==="linux")) {
-
-                    if (k===0) {
-                        var shortcuts = a.shortcuts();
-
-                        var scObjs = appWin.property("DelegatedShortcutsObjs");
-                        if (isNull(scObjs)) {
-                            scObjs=[];
-                        }
-                        for (var si=0; si<shortcuts.length; si++) {
-                            var scObj = new QShortcut(shortcuts[si], a.parentWidget(), 0, 0, Qt.WindowShortcut)
-                            scObj.activated.connect(a, "trigger");
-                            scObjs.push(scObj);
-                        }
-                        appWin.setProperty("DelegatedShortcutsObjs", scObjs);
-
-                        var sc = a.shortcut.toString();
-                        if (sc.length>0) {
-                            if (RS.getSystemId()==="linux") {
-                                a.setShortcutText("[" + sc + "]");
-                            }
-                            else {
-                                a.setShortcutText(" " + sc);
-                            }
-                        }
-                        a.setProperty("DelegatedShortcuts", shortcuts);
-                        a.setDefaultShortcuts([]);
-                    }
-                }
-                */
-
                 if (visibility) {
                     RGuiAction.addToWidget(a, w);
                     addedToWidget = true;
@@ -3295,6 +3353,12 @@ function makeQAbstractItemViewEditTriggers() {
     return makeFlags.apply(this, argumentsNew);
 }
 
+function makeQDirIteratorIteratorFlags() {
+    var argumentsNew = [].slice.call(arguments, 0);
+    argumentsNew.unshift(QDirIterator.IteratorFlags);
+    return makeFlags.apply(this, argumentsNew);
+}
+
 function makeFlags() {
     if (RSettings.isQt(6)) {
         var ret = 0;
@@ -3452,6 +3516,15 @@ function openFiles(args, createNew, close) {
     }
 }
 
+function createValidator(rx, parent) {
+    if (RSettings.getQtVersion()>=0x060000) {
+        return new QRegularExpressionValidator(new QRegularExpression(rx), parent);
+    }
+    else {
+        return new QRegExpValidator(new RegExp(rx), parent);
+    }
+}
+
 // fix QPlainTextEdit API for Qt 5:
 if (!isFunction(QPlainTextEdit.prototype.toPlainText)) {
     QPlainTextEdit.prototype.toPlainText = function() {
